#ifndef MALACHSCRIPT_MEMORYPOOL_HPP
#define MALACHSCRIPT_MEMORYPOOL_HPP

#include <cstddef>
#include <cstdint>
#include <stdexcept>

namespace MalachScript::Utils {
    template <size_t initialSize, size_t steps> class MemoryPool {
    public:
        MemoryPool() {
            auto* ptr = malloc(initialSize);
            if (ptr == nullptr) {
                throw std::logic_error("Out of memory.");
            }
            _head = static_cast<uint8_t*>(ptr);
            _tail = _head;
            _end = _head + initialSize;
        }

        ~MemoryPool() { free(_head); }

        template <class T, class... parameters> T* Create(parameters... args) {
            auto* p = _tail + sizeof(T);
            if (p >= _end) {
                IncreaseStep();
            }
            auto* element = new (_tail) T(args...);
            _tail = p;
            return element;
        }

    private:
        uint8_t* _tail = nullptr;
        uint8_t* _end;
        uint8_t* _head = nullptr;

        void IncreaseStep() {
            _end += steps;
            auto newPtr = realloc(_head, _end - _head);
            if (newPtr == nullptr) {
                throw std::logic_error("Out of memory.");
            }
            _head = static_cast<uint8_t*>(newPtr);
        }
    };
}

#endif // MALACHSCRIPT_MEMORYPOOL_HPP
